/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.adaptrex.core.ext;

import java.util.HashMap;
import java.util.Map;

import com.adaptrex.core.persistence.api.ORMModelInstance;
import com.adaptrex.core.persistence.api.ORMPersistenceManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ModelInstance {

	private static Logger log = LoggerFactory.getLogger(ModelInstance.class);

	private Map<String, Object> data = new HashMap<String, Object>();

	/*
	 * Create the model instance
	 */
	public ModelInstance(ExtConfig extConfig) {
		this(extConfig, null, null, null);
	}
	
	public ModelInstance(ExtConfig extConfig, String key, Object value) {
		this(extConfig, null, key, value);
	}

	public ModelInstance(ExtConfig extConfig, Object id) {
		this(extConfig, id, null, null);
	}

	private ModelInstance(ExtConfig extConfig, Object id, String key, Object value) {
		ORMPersistenceManager orm = extConfig.getORMPersistenceManager();
		
		Object entity;
		if (id != null) {
			entity = orm.getEntity(extConfig.getEntityClass(), id);
		} else {
			entity = orm.getEntity(extConfig.getEntityClass(), key, value);
		}
		
		ORMModelInstance ormModelInstance = orm.getModelInstance(extConfig, entity);
		this.data = ormModelInstance.getData();
	}
	
	public Map<String, Object> getData() {
		return this.data;
	}
}

