package com.adaptrex.core.persistence.jpa;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.ext.ExtConfig;
import com.adaptrex.core.ext.Filter;
import com.adaptrex.core.ext.Sorter;
import com.adaptrex.core.persistence.api.BaseStoreData;
import com.adaptrex.core.persistence.api.ORMModelInstance;
import com.adaptrex.core.persistence.api.ORMStoreData;
import com.adaptrex.core.services.AdaptrexServices;

public class JPAStoreData extends BaseStoreData implements ORMStoreData {
 
	private ExtConfig extConfig;
	
	private static Logger log = LoggerFactory.getLogger(JPAStoreData.class);
	
	@SuppressWarnings("unchecked")
	public JPAStoreData(ExtConfig extConfig) {
		this.extConfig = extConfig; 
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public List<Map<String, Object>> getData() {
		List<Map<String, Object>> data = new ArrayList<Map<String, Object>>();
		
		EntityManager em = null;
		try {
			em = (EntityManager) extConfig.getORMPersistenceManager().getSession();

			Map<String, Object> parameters = new HashMap<String, Object>(
					this.getParams());

			Class<?> clazz = extConfig.getEntityClass();
			String className = clazz.getSimpleName();
			String alias = className.toLowerCase().substring(0, 1);

			/*
			 * Core JPQL
			 */
			String jpql = "SELECT " + alias + " FROM " + className + " "
					+ alias;

			/*
			 * Where
			 */
			String where = "";
			if (this.getWhere() != null) {
				where += this.getWhere();
			}

			/*
			 * Filter
			 */
			for (Filter filter : this.getFilters()) {
				if (!where.isEmpty()) {
					where += " AND ";
				}

				String val = String.valueOf(filter.getValue());
				String prop = filter.getProperty();
				String param = prop.replace(".", "_");

				/*
				 * Handle between inclusive
				 */
				if (val.contains(">=<")) {
					String[] filterParts = val.split(">=<");

					where += "(" + alias + "." + prop + " BETWEEN :" + param
							+ "_low AND :" + param + "_high)" + "";
					parameters.put(param + "_low", filterParts[0]);
					parameters.put(param + "_high", filterParts[1]);

					/*
					 * Handle "OR"
					 */
				} else if (val.contains("||")) {
					String whereOr = "";
					int i = 1;

					for (String orPart : val.split("\\|\\|")) {
						whereOr += whereOr.isEmpty() ? "(" : " OR ";
						whereOr += alias + "." + prop + " = :" + param + i;
						parameters.put(param + i, orPart);
						i++;
					}
					whereOr += ")";
					where += whereOr;

				} else {
					String eqType = val.contains("%") ? " like " : " = ";
					if (eqType.contains("like")) {
						where += "lower(" + alias + "." + prop + ")" + eqType
								+ ":" + param;

					} else {
						where += alias + "." + prop + eqType + ":" + param;
					}

					parameters.put(param, val);
				}
			}

			/*
			 * Add where clause
			 */
			if (!where.isEmpty()) {
				jpql += " WHERE " + where;
			}

			if (this.getSorters().size() > 0) {
				String sortClause = "";
				for (Sorter sorter : this.getSorters()) {
					if (!sortClause.isEmpty()) {
						sortClause += ",";
					}
					sortClause += alias + "." + sorter.getProperty() + " "
							+ sorter.getDirection();
				}
				jpql += " ORDER BY " + sortClause;
			}

			log.debug(jpql);
			Query q = em.createQuery(jpql, clazz);

			/*
			 * Add params TODO: This is tedius... we want to automatically
			 * determine the type for the field we're trying to set the
			 * parameter for
			 */
			for (String key : parameters.keySet()) {
				Object paramObj = parameters.get(key);

				Class<?> paramType = q.getParameter(key).getParameterType();
				String typeName = paramType.getSimpleName().toLowerCase();

				try {
					if (typeName.equals("boolean")) {
						String stringVal = (String) paramObj;
						paramObj = stringVal.equals("1")
								|| stringVal.equals("true")
								|| stringVal.equals("Y");

					} else if (typeName.equals("date")) {
						paramObj = dateFormat.parse((String) paramObj);

					} else {
						Method m = paramType.getDeclaredMethod("valueOf",
								String.class);
						paramObj = m.invoke(null, paramObj);
					}
				} catch (Exception e) {
				}

				q.setParameter(key, paramObj);
			}

			this.setTotalCount(q.getResultList().size());

			if (this.getLimit() != null) {
				q.setMaxResults(this.getLimit());
			}
			if (this.getStart() != null) {
				q.setFirstResult(this.getStart());
			}
			
			for (Object entity : q.getResultList()) {
				JPAModelInstance instance = new JPAModelInstance(extConfig, entity, em);
				data.add(instance.getData());
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			em.close();
		}
		
		return data;
	}

}
